@@ -1,3 +1,4 @@
+from avrogen.core_writer import find_type_of_default
 import collections
 import six
 
@@ -6,10 +7,12 @@ from .dict_wrapper import DictWrapper
 from avro import schema
 from avro import io
 
-if six.PY3:
-    io_validate = io.Validate
-else:
-    io_validate = io.validate
+io_validate = io.validate
+
+try:
+    from avro.io import SchemaResolutionException, AvroTypeException
+except ImportError:
+    from avro.errors import SchemaResolutionException, AvroTypeException
 
 _PRIMITIVE_TYPES = set(schema.PRIMITIVE_TYPES)
 
@@ -49,8 +52,7 @@ class AvroJsonConverter(object):
                 # Match the type based on the declared schema.
                 data_schema = self._get_record_schema_if_available(datum)
                 for i, candidate_schema in enumerate(expected_schema.schemas):
-                    if candidate_schema.namespace == data_schema.namespace \
-                        and candidate_schema.name == data_schema.name:
+                    if candidate_schema.fullname == data_schema.fullname:
                         return self.validate(candidate_schema, datum)
 
             # If the union type is using a "name" to distinguish the type, we
@@ -96,8 +98,8 @@ class AvroJsonConverter(object):
         if writers_schema is None:
             raise Exception('At least one schema must be specified')
 
-        if not io.DatumReader.match_schemas(writers_schema, readers_schema):
-            raise io.SchemaResolutionException('Could not match schemas', writers_schema, readers_schema)
+        if not writers_schema.match(readers_schema):
+            raise SchemaResolutionException('Could not match schemas', writers_schema, readers_schema)
 
         return self._generic_from_json(json_obj, writers_schema, readers_schema)
 
@@ -110,13 +112,13 @@ class AvroJsonConverter(object):
         assert isinstance(writers_schema, schema.Schema)
 
         if not self.validate(writers_schema, data_obj):
-            raise io.AvroTypeException(writers_schema, data_obj)
+            raise AvroTypeException(writers_schema, data_obj)
 
         return self._generic_to_json(data_obj, writers_schema)
 
     def _fullname(self, schema_):
         if isinstance(schema_, schema.NamedSchema):
-            return schema_.fullname if six.PY2 else schema_.fullname.lstrip('.')
+            return schema_.fullname.lstrip('.')
         return schema_.type
 
     def _get_record_schema_if_available(self, data_obj):
@@ -202,8 +204,7 @@ class AvroJsonConverter(object):
         data_schema = self._get_record_schema_if_available(data_obj)
         for i, candidate_schema in enumerate(writers_schema.schemas):
             # Check for exact matches first.
-            if data_schema and candidate_schema.namespace == data_schema.namespace \
-                and candidate_schema.name == data_schema.name:
+            if data_schema and candidate_schema.fullname == data_schema.fullname:
                 index_of_schema = i
                 break
 
@@ -213,7 +214,7 @@ class AvroJsonConverter(object):
                 if candidate_schema.type == 'boolean':
                     break
         if index_of_schema < 0:
-            raise io.AvroTypeException(writers_schema, data_obj)
+            raise AvroTypeException(writers_schema, data_obj)
         candidate_schema = writers_schema.schemas[index_of_schema]
         if candidate_schema.type == 'null':
             return None
@@ -233,9 +234,9 @@ class AvroJsonConverter(object):
         if (writers_schema.type not in ['union', 'error_union']
             and readers_schema.type in ['union', 'error_union']):
             for s in readers_schema.schemas:
-                if io.DatumReader.match_schemas(writers_schema, s):
+                if writers_schema.match(s):
                     return self._generic_from_json(json_obj, writers_schema, s)
-            raise io.SchemaResolutionException('Schemas do not match', writers_schema, readers_schema)
+            raise SchemaResolutionException('Schemas do not match', writers_schema, readers_schema)
 
         result = None
         if writers_schema.type == 'null':
@@ -328,8 +329,10 @@ class AvroJsonConverter(object):
             return self._make_type(self.schema_types[readers_name], decoded_record)
         return decoded_record
 
-    def _record_from_json(self, json_obj, writers_schema, readers_schema):
-        writer_fields = writers_schema.fields_dict if six.PY2 else writers_schema.field_map
+    def _record_from_json(self, json_obj, writers_schema, readers_schema, fail_on_extra_fields=False):
+        writer_fields = writers_schema.fields_dict
+
+        input_keys = set(json_obj.keys())
 
         result = {}
         for field in readers_schema.fields:
@@ -340,9 +343,18 @@ class AvroJsonConverter(object):
             else:
                 if field.name in json_obj:
                     field_value = self._generic_from_json(json_obj[field.name], writers_field.type, field.type)
+                    input_keys.remove(field.name)
                 else:
-                    field_value = self._generic_from_json(writers_field.default,
-                                                          writers_field.type, field.type) \
-                        if writers_field.has_default else None
+                    _, nullable = find_type_of_default(field.type)
+                    if writers_field.has_default:
+                        field_value = self._generic_from_json(writers_field.default,
+                                                            writers_field.type, field.type)
+                    elif nullable:
+                        field_value = None
+                    else:
+                        raise ValueError(f'{readers_schema.fullname} is missing required field: {field.name}')
             result[field.name] = field_value
+        if input_keys and fail_on_extra_fields:
+            # only throw errors if there are fields that we do not know about and fail_on_extra_fields is set to True
+            raise ValueError(f'{readers_schema.fullname} contains extra fields: {input_keys}')
         return self._instantiate_record(result, writers_schema, readers_schema)
